package jp.co.tecnosdsm.statusbelt.cli;

import com.google.gson.Gson;
import jp.co.tecnosdsm.statusbelt.model.Anomaly;
import jp.co.tecnosdsm.statusbelt.model.ApiResult;
import jp.co.tecnosdsm.statusbelt.model.Spectrum;
import org.kohsuke.args4j.CmdLineException;
import org.kohsuke.args4j.CmdLineParser;
import org.sql2o.Connection;
import org.sql2o.Sql2o;

import java.text.SimpleDateFormat;
import java.util.List;

import static spark.Spark.*;

/**
 */
public class Bootstrap {
    private static final String JDBC_URL_TEMPLATE = "jdbc:sqlserver://%s:1433;database=%s;encrypt=true;trustServerCertificate=false;hostNameInCertificate=*.database.windows.net;loginTimeout=30;";

    public static void main(String[] args) {

        // Parse Arguments
        CommandParameter param = new CommandParameter();
        CmdLineParser parser = new CmdLineParser(param);

        try {
            parser.parseArgument(args);

        } catch (CmdLineException e) {
            parser.printUsage(System.out);
            System.exit(1);
        }

        ////////////////////////////
        // Setting
        ////////////////////////////

        port(param.getPort());

        staticFileLocation("/public");

        Gson gson = new Gson();

        Sql2o sql2o = new Sql2o(
                String.format(JDBC_URL_TEMPLATE, param.getDbHostname(), param.getDbName())
                , param.getDbUser(), param.getDbPassword());

        ////////////////////////////
        // Action
        ////////////////////////////

        // Index Page
        get("/", (request, response) -> {
            response.redirect("/index.html");
            return "";
        });

        // API
        String anomalySql = "SELECT TOP 1 recorded_at, anomaly_score, is_anomaly FROM anomaly ORDER BY recorded_at DESC";
        String spectrumSql = "SELECT recorded_at, x, y FROM spectrum WHERE recorded_at = :recorded_at ORDER BY x";

        get("/api/spectrum", (request, response) -> {

            try(Connection con = sql2o.open()) {
                Anomaly anomaly = con.createQuery(anomalySql)
                        .setAutoDeriveColumnNames(true)
                        .executeAndFetchFirst(Anomaly.class);

                if (anomaly == null) {
                    return ApiResult.fail();
                }

                List<Spectrum> spectrums = con.createQuery(spectrumSql)
                        .setAutoDeriveColumnNames(true)
                        .addParameter("recorded_at", anomaly.getRecordedAt())
                        .executeAndFetch(Spectrum.class);

                if (spectrums.isEmpty()) {
                    return ApiResult.fail();
                }

                anomaly.setRecordedAtStr(new SimpleDateFormat("yyyy/MM/dd HH:mm:ss").format(anomaly.getRecordedAt()));

                return new ApiResult(true, anomaly, spectrums);
            }

        }, gson::toJson);
    }

}
